<?php

namespace App\domain\travelport;

use Carbon\Carbon;
use Carbon\CarbonInterval;
use Exception;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Log;
use Spatie\Async\Pool;

class QueryCombinableFlightOffersAction
{
    protected string $transactionId;
    protected Collection $referenceList;

    public function __construct(protected FlightOfferService $flightOfferService)
    {
    }

    public function execute(array $passengerCriteria, array $searchCriteriaFlight, ?array $searchModifiers): array
    {
        $numberOfLegs = count($searchCriteriaFlight);

//        $ndcCombinableOffers = $this->getCombinableOffers(
//            "NDC", $passengerCriteria, $searchCriteriaFlight, $searchModifiers);
//
//        $gdsCombinableOffers = $this->getCombinableOffers(
//            "GDS", $passengerCriteria, $searchCriteriaFlight, $searchModifiers);
//
//        $totalCombinableOffers = [...$ndcCombinableOffers, ...$gdsCombinableOffers];

        $pool = Pool::create();

        $pool->add(function () use ($passengerCriteria, $searchCriteriaFlight, $searchModifiers) {
            return $this->getCombinableOffers("NDC", $passengerCriteria, $searchCriteriaFlight, $searchModifiers);
        });

        $pool->add(function () use ($passengerCriteria, $searchCriteriaFlight, $searchModifiers) {
            return $this->getCombinableOffers("GDS", $passengerCriteria, $searchCriteriaFlight, $searchModifiers);
        });

        $results = $pool->wait();

        $totalCombinableOffers = array_merge(...$results);

        if ($numberOfLegs === 1) {
            return $totalCombinableOffers;
        }

        return (new Collection($totalCombinableOffers))
            ->filter(fn($combinableOffer) => count($combinableOffer) > 1)
            ->toArray();
    }

    protected function getCombinableOffers(
        string $source,
        array  $passengerCriteria,
        array  $searchCriteriaFlight,
        ?array $searchModifiers
    ): array
    {
        $ndcContents = $this->flightOfferService->requestOffers(
            $passengerCriteria,
            $searchCriteriaFlight,
            $searchModifiers,
            $source
        );

        return $this->generateCombinableOffers($ndcContents);
    }

    protected function generateCombinableOffers(array $offersContents): array
    {
        $catalogProductOfferings = new Collection(
            collect($offersContents['CatalogProductOfferingsResponse']['CatalogProductOfferings'])
        );
        $this->transactionId = $catalogProductOfferings['Identifier']['value'];
        $catalogProductOffering = new Collection($catalogProductOfferings['CatalogProductOffering']);
        $this->referenceList = new Collection(collect($offersContents['CatalogProductOfferingsResponse']['ReferenceList']));

        $productOffers = $catalogProductOffering->flatMap(function ($offer) {
            return collect($offer['ProductBrandOptions'])->flatMap(
                function ($productOption) use ($offer) {

                    return $this->generateBrandOptions($offer, $productOption);
                });
        });

        return $productOffers->groupBy('combinabilityCode')->toArray();
    }

    protected function generateBrandOptions($offer, $productOption): Collection
    {
        return collect($productOption['ProductBrandOffering'])->flatMap(
            function ($productBrandOffering) use ($offer) {

                return $this->generateProductBrands($productBrandOffering, $offer);
            });
    }

    protected function generateProductBrands($productBrandOffering, $offer): Collection
    {
        return collect($productBrandOffering['CombinabilityCode'])->map(
            function ($combinabilityCode) use ($offer, $productBrandOffering) {

                return $this->generateOfferProduct($combinabilityCode, $offer, $productBrandOffering);
            });
    }

    protected function generateOfferProduct($combinabilityCode, $offer, $productBrandOffering): array
    {
        $productIAir = $this->extractProductAirFromReferenceList(
            $productBrandOffering['Product'][0]['productRef']
        );

        $flightSegments = new Collection($productIAir['FlightSegment']);

        $flightSegments = $this->addFlightDetailsToSegments($flightSegments);

        $segmentDetails = $this->extractFlightSegmentDetails($flightSegments);

        $durations = $this->calculateProductDurations($productIAir);

        return [
            '@type' => intval($offer['sequence']) === 1 ? 'Outbound' : 'Inbound',
            'combinabilityCode' => $combinabilityCode,
            'catalogProductOfferingsIdentifier' => $this->transactionId,
            'authority' => $offer['Identifier']['authority'] ?? "GDS",
            'offerId' => $offer['id'],
            'id' => $productBrandOffering['Product'][0]['productRef'],
            'fromAirport' => $offer['Departure'],
            'fromAirportCarrierCode' => $segmentDetails['fromAirportCarrierCode'],
            'fromAirportAircraftCode' => $segmentDetails['fromAirportAircraftCode'],
            'toAirport' => $offer['Arrival'],
            'toAirportCarrierCode' => $segmentDetails['toAirportCarrierCode'],
            'toAirportAircraftCode' => $segmentDetails['toAirportAircraftCode'],
            'originDepartureTime' => $segmentDetails['originDepartureTime'],
            'destinationArrivalTime' => $segmentDetails['destinationArrivalTime'],
            'duration' => $durations['Duration'],
            'numberOfStops' => $flightSegments->count() - 1,
            'numberOfBookableSeats' => $productIAir['Quantity'] ?? 0,
            'segments' => $flightSegments->toArray(),
            'durationInHours' => $durations['Hours'],
            'price' => $productBrandOffering['Price'],
            'bestCombinablePrice' => $productBrandOffering['BestCombinablePrice'],
            'productAir' => $productIAir,
        ];
    }

    protected function extractProductAirFromReferenceList(string $productRef)
    {
        $productAirList = new Collection(
            $this->referenceList->where("@type", "ReferenceListProduct")->first()["Product"]
        );

        return $productAirList->where("id", $productRef)->first();
    }

    protected function addFlightDetailsToSegments(Collection $flightSegments): Collection
    {
        $referenceFlights = new Collection(
            $this->referenceList->where("@type", "ReferenceListFlight")->first()['Flight']
        );

        return $flightSegments->map(function ($segment) use ($referenceFlights) {
            $flightDetails = $referenceFlights->where('id', $segment['Flight']['FlightRef'])->first();

            $segment['FlightDetails'] = $flightDetails;

            return $segment;
        });
    }

    protected function extractFlightSegmentDetails(Collection $segments): array
    {
        $duration = $this->calculateDirectionDuration($segments);

        $firstFlight = Collection::wrap($segments->first()["FlightDetails"]);
        $lastFlight = Collection::wrap($segments->last()["FlightDetails"]);
        return [
            'originDepartureTime' => $this->getDepartureOrArrivalTime($firstFlight, direction: "Departure"),
            'destinationArrivalTime' => $this->getDepartureOrArrivalTime($lastFlight, direction: "Arrival"),
            'duration' => $duration['Duration'],
            'durationInHours' => $duration['Hours'],
            'fromAirportCarrierCode' => $firstFlight['carrier'],
            'toAirportCarrierCode' => $lastFlight['carrier'],
            'fromAirportAircraftCode' => $firstFlight['equipment'],
            'toAirportAircraftCode' => $lastFlight['equipment'],
            'fromAirline' => isset($lastFlight['operatingCarrierName'])
                ? $lastFlight['operatingCarrierName']
                : "",
        ];
    }

    protected function calculateProductDurations(array $productAir): array
    {
        try {
            $totalHours = 0;
            $totalMinutes = 0;

            $totalDuration = CarbonInterval::fromString($productAir["totalDuration"]);

            $totalHours += $totalDuration->hours;
            $totalMinutes += $totalDuration->minutes;

            $totalHours += floor($totalMinutes / 60);
            $totalMinutes = $totalMinutes % 60;

            $durationHours = $totalHours + ($totalMinutes ? $totalMinutes / 60 : 0);

            return ["Duration" => $productAir["totalDuration"], "Hours" => $durationHours];

        } catch (Exception $e) {
            return ["Duration" => "", "Hours" => ""];
        }
    }

    protected function calculateDirectionDuration(Collection $segments): array
    {
        try {
            $durations = $segments->pluck('FlightDetails.duration');

            $totalHours = 0;
            $totalMinutes = 0;

            $durations->each(function ($durationString) use (&$totalHours, &$totalMinutes) {
                $duration = CarbonInterval::fromString($durationString);
                $totalHours += $duration->hours;
                $totalMinutes += $duration->minutes;
            });

            $totalHours += floor($totalMinutes / 60);
            $totalMinutes = $totalMinutes % 60;

            $durationHours = $totalHours + ($totalMinutes ? $totalMinutes / 60 : 0);

            return ["Duration" => "PT{$totalHours}H{$totalMinutes}M", "Hours" => $durationHours];

        } catch (Exception $e) {
            return ["Duration" => "", "Hours" => ""];
        }
    }

    private function getDepartureOrArrivalTime(Collection $firstFlight, string $direction): string
    {
        $departureDate = $firstFlight->get($direction)['date'];
        $departureTime = $firstFlight->get($direction)['time'];

        return Carbon::createFromFormat('Y-m-d H:i:s', $departureDate . ' ' . $departureTime)->toISOString();
    }
}
